
#include <stdio.h>
#include <termios.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/stat.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <dirent.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <pwd.h>
#include <grp.h>
#include <string.h>
#include <stdlib.h>
#include <devlock.h>
#include <linux/serial.h>

extern int devunlock( char * );
extern int devlock( char * );
char  act_device[64];

int debug;
struct termios old, raw;

void usage(void);
char raw_char(void);
int show_char(unsigned char, int);
int catch_line(char *);
void print_lines( struct serial_icounter_struct * );


int
main(int argc, char *argv[])
{
	time_t now;
	char *p;
	int i;

	i = 1;
	
	while (i <= argc) {
		if (!(p = argv[i++]))
			break;
		if (*p == '-') {
			switch (*++p) {
			case 'd':
			case 'x':				  /* set the debug level */
				if (!*++p)
					p = argv[i++];
				debug = atoi(p);
				break;
			}
		} else if (*p == '?') {
			usage();
		} else strncpy( act_device, p, sizeof( act_device ));
	}
#if 0
	while ( devlock( act_device ) != 0) {
		if (debug) {
			now = time(0L);
			fprintf(stderr, "%.24s: %s locks are set. Sleeping...\n",
					  ctime(&now), act_device);
		}
		sleep(30);
	}
#endif
	ioctl(0, TCGETS, &old);
	raw = old;
	raw.c_cc[VMIN] = 0;
	raw.c_cc[VTIME] = 0;
	raw.c_lflag = old.c_lflag & ~(ICANON | ECHO);
	catch_line(act_device);
	/*	devunlock( act_device ); */
	time(&now);
	fprintf(stderr, "%.24s: %s Exiting\n", ctime(&now), act_device);
	exit(0);
}

void
usage()
{
	fprintf(stdin, "check: -s <speed>\n");
	fprintf(stdin, "       -x debug level\n");
	fprintf(stdin, "        <port>\n");
	exit(0);
}

char
raw_char()
{
	char ch;

	ioctl(0, TCSETS, &raw);
	ch = getchar();
	ioctl(0, TCSETS, &old);
	return(( ch >= ' ' && ch <= '~' ) ? ch : '\0' );
}

/*
 * Put a char on stderr formatted as ^ or M- as required:
 * which == 0 put a char
 * which == 1 accumlate a line and insert a \n
 */
int
show_char(unsigned char ch, int which)
{
	int count = 0;
	static int total;
	char buf[80], *p;

	p = buf + total;
	if (ch > 0x7f) {
		*p++ = 'M';
		*p++ = '-';
		ch &= 0x7f;
	}
	if (ch < ' ') {
		if (ch == '\n')
			*p++ = ch;
		else {
			*p++ = '^';
			*p++ = ch + 0x40;
		}
	} else if (ch < 0x7F)
		*p++ = ch;
	else {
		*p++ = '^';
		*p++ = '?';
	}
	count = (p - buf) - total;
	if (which) {
		if ((ch == '\n') || ((total += count) >= 77)) {
			total = 0;
			*p++ = '\n';
			*p = '\0';
			fputs(buf, stderr);
		}
	} else {
		/* while( count++ < 4 ) *p++ = ' '; */
		*p = '\0';
		fputs(buf, stderr);
	}
	return (count);
}

int
catch_line( char *fname )
{
    int flag, ret;
    long f_flag;
	time_t now;
    int fd;
    struct termios old_termios, new_termios;
	struct serial_icounter_struct which;

    if (( fd = open( fname, O_RDONLY | O_NONBLOCK, &flag )) < 0 ) {
        fprintf( stderr, "failed to open %s\n%s\n", fname,
                 sys_errlist[errno] );
        return( -1 );
    }
    /* We had to open() with O_NONBLOCK set since we don't know the
     * status of DCD.  Now set the driver back to blocking for the
     * reads we will do later unless you know that the packets are
     * terminated with a new line. */
    if ( fcntl( fd, F_GETFL, &f_flag ) < 0 ) {
        fprintf( stderr, "failed to get fcntl flags.\n%s\n",
                 sys_errlist[errno] );
        return( -1 );
    }
    f_flag &= ~O_NONBLOCK;
    if ( fcntl( fd, F_SETFL, &f_flag ) < 0 ) {
        fprintf( stderr, "failed to set fcntl flags.\n%s\n",
                 sys_errlist[errno] );
        return( -1 );
    }
        
    ioctl( fd, TCGETS, &old_termios );
    /* Here you can set the various struct termios flags as defined
     * in 'termios(7)' if you can find a copy.  Otherwise read the
     * termios.blurb in my "serial_suite".  For instance, you may want 
     * to set c_cc[VMIN] to the size of the packet if you know it and
     * c_cc[VTIM] = 0.
     * The CLOCAL, READ, and ICANON flags should be set acording to
     * how the packet is formatted. */
    old_termios.c_cflag &= CBAUD;
    old_termios.c_cflag |= B19200;
    ioctl( fd, TCSETS, &old_termios );
    ioctl( fd, TCGETS, &new_termios );
    if ( memcmp( &old_termios, &new_termios, sizeof( struct termios )) != 0 ) {
        printf( "Writing changes to termios failed.\n" );
        return( -1 );
    }
	ioctl( fd, TIOCGICOUNT, &which );
	print_lines( &which );
    flag = TIOCM_RNG | TIOCM_DSR | TIOCM_CD | TIOCM_CTS;
	while( 1 ) {
		time(&now);
		printf( "%.024s Wait.\n", ctime(&now));
		
		if (( ret = ioctl( fd, TIOCMIWAIT, &flag )) != 0 ) {
			close( fd );
			if (( fd = open( fname, O_RDONLY | O_NONBLOCK, &flag )) < 0 ) {
				printf( "failed to open %s\n%s\n", fname,
						 sys_errlist[errno] );
				break;
			}
			printf( "Reopen, %s\n", sys_errlist[errno] );
		}
		
    /* My impression is that this ioctl() will block until a ring is
     * detected.  If any of the other options are or'ed into 'flag'
     * they will cause the ioctl() to return when any one of them is 
     * detected.  I don't know how you can determine which one caused
     * the call to return. */
		time(&now);
		printf( "%.024s Read.\n", ctime(&now));
		ioctl( fd, TIOCGICOUNT, &which );
		print_lines( &which );
	}
	return( 0);
}

void
print_lines( struct serial_icounter_struct *which )
{
	static struct serial_icounter_struct prev;

	printf( "\tCTS %d %d, ", which->cts, which->cts - prev.cts);
	printf( "DSR %d %d, ", which->dsr, which->dsr - prev.dsr);
    printf( "RNG %d %d, ", which->rng, which->rng - prev.rng );
    printf( "DCD %d %d\n", which->dcd, which->dcd - prev.dcd );
#if 0	/* available in 2.2.x */
    printf( "\tRX  %d, ", which->rx  );
	printf( "TX  %d\n", which->tx  );
	printf( "\tFRAME %d, ", which->frame );
	printf( "OVERRUN %d, ", which->overrun );
	printf( "PARITY  %d, ", which->parity  );
    printf( "\tBREAK   %d\n", which-> brk    );
	printf( "\tBUFFER  %d\n", which->buf_overrun );
#endif
	prev.cts = which->cts;
	prev.dsr = which->dsr;
	prev.rng = which->rng;
	prev.dcd = which->dcd;
	
	return;
}

    
