#include "common.h"
#include "gettydefs.h"
#include <ctype.h>

#define RECVFAX       "/usr/bin/netfax"	  /* Command line for receiving
                                           * a fax */
#define LOGIN         "/bin/login"
#define LOGFILE       "/var/log/getty_em" /* Where errors will be logged */
#define CHECK_GETTY   "/usr/lib/uucp/check_em"

extern int   set_speed(int, enum how);
extern int   read_defs(char *, char *);
extern char *read_response(int);
extern void  display_strg(unsigned char *);
extern char *display_char(unsigned char);
extern int   open_device(char *);
extern int   devlock( char * );
extern int   devunlock( char * );
extern char  act_device[];
extern struct defs *defs_base;

struct termios old, raw;
int debug;

void catch_int(int);
void catch_hup(int);
void catch_alarm(int);
int  do_wait(int, int);
int  get_login(int, char *);
void usage(void);
int  get_type(char *);
/* int  do_fax(int); */
int  is_device( char * );

int
main(int argc, char *argv[])
{
	time_t now;
	char *p, *fname;
	char getty_entry[30];
	int time_out;
	int two_way;
	int check_it;
	int i, fd;

	act_device[0] = '\0';
	strncpy( getty_entry, "T19200", 30 );	/* Default value */
	fname = DEFS_FILE;
	two_way = FALSE;
	check_it = FALSE;
	time_out = 60;
	i = 1;
	while (i <= argc) {
		if (!(p = argv[i++]))
			break;
		if (*p == '-') {
			switch (*++p) {
			case 'c':				  /* just check the 'gettydefs' file */
				check_it = TRUE;
				if (!*++p)
					p = argv[i++];
				if ((access(p, F_OK)) == 0)
					fname = p;
				break;
			case 'r':				  /* two way communication on this line */
				two_way = TRUE;
				break;
			case 't':				  /* set the timeout interval */
				if (!*++p)
					p = argv[i++];
				time_out = atoi(p);
				break;
			case 'd':
			case 'x':				  /* set the debug level */
				if (!*++p)
					p = argv[i++];
				debug = atoi(p);
				break;
			}
		} else if (*p == '?')
			usage();
		else if ( is_device( p ));
		else
			strncpy(getty_entry, p, sizeof(getty_entry));
	}
	if (getppid() <= 1) {
		freopen(LOGFILE, "a", stderr);
		setvbuf(stderr, NULL, _IONBF, 0);
		time(&now);
		/*		fflush( stderr ); */
		fprintf(stderr, "%.24s: %s %d, Startup\n",
				ctime(&now), act_device, getpid());
	}
	
	if ( read_defs( getty_entry, fname ) == -1 ) {
		fprintf( stderr, "Bad gettydefs entry, %s\n", getty_entry );
		exit( -1 );
	}
	if (check_it) {
		printf("time_out: %d\n", time_out);
		printf("two_way: %s\n", two_way ? "TRUE" : "FALSE");
		printf("check_it: %s\n", check_it ? "TRUE" : "FALSE");
		printf("file name: %s\n\n", fname);
		printf("device: %s\n", act_device );
		printf("entry requested: %s\n", getty_entry);
		execl(CHECK_GETTY, fname);
		exit(0);
	}
	/* 
	 * This call to 'open_device' also calls 'ttylock'.  This will
	 * clear any abandond lockfiles.  If there is an active lock,
	 * we stall here until the device is released.
	 */
	while (( fd = open_device( act_device )) < 0) {
		if (debug) {
			now = time(0L);
			fprintf(stderr, "%.24s: %s %d Sleeping main ...\n",
					  ctime(&now), act_device, getpid());
		}
		sleep(30);
	}

	/* Start the getty and wait for a connection. */
	do_wait( fd, time_out );
	if (debug) {
		time(&now);
		fprintf(stderr, "%.24s: %s Exiting...\n",
				  ctime(&now), act_device );
	}
	close(fd); 		/* Close the device */
	
	exit(0);
}

void
catch_hup(int sig)
{
	time_t now;

	time(&now);
	fprintf(stderr, "%.24s: %s Caught HANGUP\n", ctime(&now), act_device);
	exit(-1);
}

void
catch_int(int sig)
{
	time_t now;

	time(&now);
	fprintf(stderr, "%.24s: %s Caught INTERRUPT\n", ctime(&now), act_device);
	exit(-1);
}

void
catch_alarm(int sig)
{
	time_t now;

	time(&now);
	fprintf(stderr, "%.24s: %s Timed out.\n", ctime(&now), act_device);
	exit(-1);
}

int
do_wait( int fd, int time_out)
{
	int cur_getty;					  /* index into gettydefs array */
	int ret, flags, back;
	int timer_on;
	unsigned char ch1, ch2, ch3;
	time_t now;

	setsid();
	if ((cur_getty = set_speed(fd, local)) < 0) {
		fprintf(stderr, "Error setting initial speed, %s\n",
				  sys_errlist[errno]);
		close(fd);
		return (-1);
	}

	/* Toggle DTR to reset modem */
	flags = TIOCM_DTR | TIOCM_RTS;
	ioctl(fd, TIOCMBIC, &flags);	/* Turn off flags */
	ioctl(fd, TCFLSH, 2); 			/* Flush both buffers */
	sleep(1);
	ioctl(fd, TIOCMBIS, &flags);	/* Turn on flags */

	signal(SIGINT, catch_int);
	/*	set_speed(fd, cooked); */
	/*	set_speed(fd, toloc); */
	timer_on = 0;
	ch2 = '\0';
	ch3 = '\0';
	back = 0;
	/*
	 * Remove the lockfiles now so that outgoing programs can
	 * open the device.
	 */
	devunlock( act_device );
	now = time(0L);
	fprintf(stderr, "%.24s: %s %d, Waiting for chars ... \n",
				  ctime(&now), act_device, getpid());
	/*
	 * This read will block waiting for incoming data since the device
	 * was opened without the NONBLOCK flag being set.
	 *
	 * This might be a place to change the process group
	 * leader to the pid given as ret.
	 */
	while ((ret = read(fd, &ch1, sizeof(ch1))) == 1) {
		if (debug)
			fputs( display_char(ch1), stderr );
		if (((ch1 == '\r') || (ch1 == '\n')) &&
					((ch2 == '\r') || (ch2 == '\n'))) {
			back = 1;
			break;
		}
		if ( ch1 == '\0' && ch2 == 0377) {
			back = 1;
			break;
		}
		if ( back != 0 ) break;
		ch3 = ch2;
		ch2 = ch1;
	}
	if (debug) {
		fputc('\n', stderr);
		fputs( "ch1 ", stderr );
		fputs( display_char(ch1), stderr );
		fputs( " ch2 ", stderr );
		fputs( display_char(ch2), stderr );
		fprintf( stderr, " back %d\n", back );
	}
	if (( ret = devlock( act_device)) > 0 ) {
		now = time(0L);
		fprintf(stderr, "%.24s: %s %d, Stalling.\n",
				ctime(&now), act_device, getpid());
		while (( ret = devlock( act_device)) > 0 ) {
			/*
			 * If we 'close' while this file descriptor is active.
			 * the kernel causes a disconnect of the modem.
			 */
			if ( debug )
				fprintf(stderr, "Lock file found\n" );
			sleep( 30 );
		}
		back = -1;
	}
	else if (back > 0) {
		if (!timer_on) {
			timer_on++;
			if (debug) {
				now = time(0L);
				fprintf(stderr, "\n%.24s: %s Start timer, %d sec.\n",
						  ctime(&now), act_device, time_out);
			}
			alarm(time_out);
		}
		signal(SIGHUP, catch_hup);
		/*
		 * At this point, we are looking for an incoming call.  So
		 * we set the lockfiles to discouraging outgoing calls
		 * from interfering.
		 */
		back = get_login(fd, (defs_base + cur_getty)->prompt);
	}
	return (back);
}

int
get_login(int fd, char *prompt)
{
	struct passwd *entry;
	static char query[] = "\nEnter Password: ";
	static char bad_word[] = "Faulty password.\n";
	int caps, lower;
	char *ret_word, *ep_word;
	char login[16];
	char buf[80];
	char *argv[10];
	char lsh[] = "login -pf ";
	char *p, *q, *path;
	time_t now, next;
	int count, len, ret, i;
	uid_t my_uid, my_gid;
	int index, stop, tries, good;

	now = time(0L);
	/*	set_speed(fd, r_raw); */
	set_speed(fd, nopar);
	set_speed(fd, onlcr);
	tries = 4;				/* Issue 'ogin:', 'sword:' prompts
							 * 4 times then quit */
	ioctl(fd, TCFLSH, 2);
 retry:
	len = strlen(prompt);
	set_speed(fd, echo);
	if (debug) {
		now = time(0L);
		fprintf(stderr, "%.24s: %s %start login.\n",
				  ctime(&now), act_device, tries == 4 ? "S" : "Res");
	}
	ret = 0;
	entry = NULL;
	next = 0;
	ret_word = NULL;
	for (count = 10; count;) {
		/* Don't issue prompt more often than 8 sec. */
		if (time(&now) >= next && ret_word == NULL ) {
			next = now + 8;
			count--;
			if( debug ) {
				fprintf(stderr, "Count %d\n", count);
				fprintf( stderr, "Sending: %s\n", prompt );
			}
			write(fd, prompt, len);
		}
		if ((ret_word = read_response(fd)) != NULL) {

			/*
			 * Make sure that the numbers on the next 2 lines correspond
			 * to the list in 'get_type'
			 */
			ret = get_type(ret_word);
#if 0
			if ( ret == 6 )
					return (do_fax(fd));
#endif
			/* We may have intruded on a call out routine. */
			if (ret == 0) {
				/* We stole their 'OK', give it back. */
				write(fd, "OK\n", 3);
				return (0);
			}
			if (ret == 7 )
					/* They are sending 'AT', get out now. */
					return (0);
			else if (ret == 8 || ret == 9 )
					continue;

			caps = 0;
			lower = 0;
			for( p = ret_word, q = p + 80; *p && p < q; p++ ) {
				if( *p >= 'A' && *p <= 'Z' ) {
					caps++;
					*p = tolower( *p );
				}
				else lower++;
			}
			if( ! lower ) {
				/* We are connecting to a dumb terminal without
				 * lower case capability. */
				set_speed( fd, dumb );
			}

			setpwent();		  /* We reset before each try */
			if ((entry = getpwnam(ret_word)) != NULL)
				break;
			if (debug) {
				now = time(0L);
				fprintf(stderr, "%.24s: %s Bad Login ret_word: '%s'\n",
						  ctime(&now), act_device, ret_word);
			}
		}
	}
	if (count <= 0) {
		fprintf(stderr, "Attempt to read login failed 10 times.\n");
		return (0);
	}
	strncpy(login, ret_word, sizeof(login));
	if (debug) {
		now = time(0L);
		fprintf(stderr, "\n%.24s: %s Login '%s'\n",
				  ctime(&now), act_device, login);
	}
	if ((strlen(entry->pw_shell) == 0) || (*entry->pw_passwd == '*')) {
		fprintf(stderr, "No shell provided for login '%s'\n", ret_word);
		return (0);
	}
	ioctl(fd, TCFLSH, 2);
	/* Now get the password if one is required. */
	if (strlen(entry->pw_passwd) != 0) {
		len = strlen(query);
		if( debug )
				fprintf(stderr, "%.24s: %s Sending: '%s'\n",
						  ctime(&now), act_device, query);
		write(fd, query, len);
		set_speed(fd, noecho);
		while ( 1 ) {
			ret_word = read_response(fd);
			if( ret_word == NULL ) continue;
			if(( ret = get_type(ret_word)) > 9 ) break;
		}
		ep_word = crypt(ret_word, entry->pw_passwd);
		if ((good = strncmp(entry->pw_passwd, ep_word,
								  strlen(entry->pw_passwd))) != 0) {
			fprintf(stderr, "\n%.24s: %s Bad Password: '%s'\n",
					  ctime(&now), act_device, ret_word);
			len = strlen( bad_word );
			write( fd, bad_word, len);
			goto retry;
		}
		write( fd, "\n", 1 );
	}
	endpwent();
	if (debug) {
		now = time(0L);
		fprintf(stderr, "\n%.24s: %s Password '%s'\n",
				  ctime(&now), act_device, ret_word);
	}
	if (entry->pw_gid == UUCP_GID) {
		my_uid = UUCP_UID;
		my_gid = UUCP_GID;
	} else {
		my_uid = entry->pw_uid;
		my_gid = entry->pw_gid;
	}
	set_speed(fd, run);
	path = LOGIN;
	p = lsh;
	strncat(strncpy(buf, p, sizeof(buf) - 1),
			  entry->pw_name, sizeof(buf) - 1);
	if (debug > 3) {
		struct group *gr_ent;

		setgrent();
		gr_ent = getgrgid(entry->pw_gid);
		fprintf(stderr,
		  "Changing owner to '%s', pid %d, and group to '%s', gid %d\n",
		entry->pw_name, entry->pw_uid, gr_ent->gr_name, gr_ent->gr_gid);
		fprintf(stderr, "Exec'ing '%s', '%s'\n", path, buf);
	}

	/* Convert the string in buf to a series of args */
	p = buf;
	q = p;
	index = 0;
	stop = 0;
	while (!stop) {
		while (*q && *q != ' ')
				q++;
		if (!*q)
				stop = 1;
		*q++ = '\0';
		argv[index++] = p;
		p = q;
	}
	argv[index] = NULL;

	for (i = 0; i < fd; i++)
			close(i);
	for (i = 0; i < 3; i++)
			dup2(fd, i);
	if (fd >= 3) {
		close(fd);
		fd = 0;
	}
	execv(path, argv);
	fprintf(stderr, "exec failed to run %s", path);
	return (-1);
}

void
usage()
{
	fprintf(stdin, "getty_em: [-r] [-x] -t device entry\n");
	fprintf(stdin, "getty_em: -c gettydefs-type-file\n");
	fprintf(stdin, "getty_em: -q device\n");
	exit(0);
}

/* Parse the response code from modems. */
int
get_type(char *word)
{
	int ret, i;
	char *msg, *p, *q, *r;
	char buf[10];

	/* Do a check for numeric response codes */
	if (*word >= '0' && *word <= '9') {
		p = word;
		q = buf;
		r = q + sizeof(buf);
		while (*p && q < r)
			*q++ = *p++;
		*q = '\0';
		return (atoi(buf));
	}

	/*
	 * This list should correspond to most Hayes compatible modems. The
	 * 'for' loop which writes the login prompt in 'get_login' should be
	 * checked for logical consistency if any changes are made here.
	 */
	ret = 0;
	for (i = 0; i <= 10; i++) {
		switch (i) {
		case 0: msg = "OK";         break;
		case 1: msg = "CONNECT";    break;
		case 2: msg = "RING";       break;
		case 3: msg = "NO CARRIER"; break;
		case 4: msg = "ERROR";      break;
		case 5: msg = "DATA";       break;
		case 6: msg = "FAX";        break;
		case 7: msg = "AT";         break;
		case 8: msg = "\n";         break;
		case 9: msg = "\r";         break;
		default: msg = "";          break;
		}
		p = word;
		while( *msg == *p && *msg ) msg++, p++;
		if ( ! *msg )
			break;
	}
	return (i);
}

int
is_device( char *name )
{
	if(access( name, F_OK) == 0) {
		strncpy( act_device, name, MAXNAMLEN );
		return( 1 );
		
	}
	else {
		strncpy( act_device, "/dev/", MAXNAMLEN );
		strncat( act_device, name,  MAXNAMLEN );
		if( access( act_device, F_OK) == 0)
			return( 1 );
	}
	act_device[0] = '\0';
	return( 0 );
}

#if 0
/* 
 * I haven't got this to work on my modem, maybe you can get it to run.
 * vch
 */
int
do_fax(int fd)
{
time_t now;
now = time(0L);
fprintf( stderr, "do_fax %.24s\nFAX detected.\n", ctime(&now));
	if (set_locks() < 0)
		return (-1);
	/*
	 * We do a system() call here so that we will return and be able to
	 * zap our lock files.
	 */
/* 	system(RECVFAX); */
	check_locks();					  /* if check_locks() finds our pid, it
										   * zaps it */
	return (0);
}
#endif
